@@ -54,7 +54,7 @@
 #define MOD_GM_COMMON_H
 
 /* constants */
-#define GM_VERSION                 "1.2.4"
+#define GM_VERSION                 "1.2.6"
 #define GM_ENABLED                      1
 #define GM_DISABLED                     0
 #define GM_BUFFERSIZE               98304
@@ -102,7 +102,7 @@
 /* worker */
 #define GM_DEFAULT_MIN_WORKER           1      /**< minumum number of worker             */
 #define GM_DEFAULT_MAX_WORKER          20      /**< maximum number of concurrent worker  */
-#define GM_DEFAULT_JOB_MAX_AGE        600      /**< discard jobs older than that         */
+#define GM_DEFAULT_JOB_MAX_AGE          0      /**< discard jobs older than that         */
 #define GM_DEFAULT_SPAWN_RATE           1      /**< number of spawned worker per seconds */
 #define GM_DEFAULT_WORKER_LOOP_SLEEP    1      /**< sleep in worker main loop */
 
@@ -202,6 +202,7 @@ typedef struct mod_gm_opt_struct {
     char         * local_servicegroups_list[GM_LISTSIZE];   /**< list of group  which will not be distributed */
     int            local_servicegroups_num;                 /**< number of elements in local_servicegroups_list */
     int            do_hostchecks;                           /**< flag whether mod-gearman will process hostchecks at all */
+    char         * queue_cust_var;                          /**< custom variable name which contains the target queue */
     mod_gm_exp_t * exports[GM_NEBTYPESSIZE];                /**< list of exporter queues */
     int            exports_count;                           /**< number of export queues */
     int            orphan_host_checks;                      /**< generate fake result for orphaned host checks */
@@ -219,6 +220,7 @@ typedef struct mod_gm_opt_struct {
     int            max_jobs;                                /**< maximum number of jobs done after a worker exits */
     int            spawn_rate;                              /**< number of spawned new worker */
     int            show_error_output;                       /**< optional display the stderr output of plugins */
+    int            dup_results_are_passive;                 /**< send duplicate results as passive checks */
 #ifdef EMBEDDEDPERL
     int            enable_embedded_perl;                    /**< enabled embedded perl */
     int            use_embedded_perl_implicitly;            /**< use embedded perl implicitly */
@@ -256,7 +258,8 @@ typedef struct gm_job_struct {
     int            exited_ok;           /**< did the plugin exit normally? */
     int            timeout;             /**< timeout for this job */
     double         latency;             /**< latency for from this job */
-    struct timeval core_start_time;     /**< time when the core started the job */
+    struct timeval next_check;          /**< next_check value of host / service */
+    struct timeval core_time;           /**< time when the core started the job */
     struct timeval start_time;          /**< time when the job really started */
     struct timeval finish_time;         /**< time when the job was finished */
 } gm_job_t;
