@@ -72,25 +72,25 @@ void worker_client(int worker_mode, int indx, int shid) {
     if(set_worker(&worker) != GM_OK) {
         gm_log( GM_LOG_ERROR, "cannot start worker\n" );
         clean_worker_exit(0);
-        exit( EXIT_FAILURE );
+        _exit( EXIT_FAILURE );
     }
 
     /* create client */
     if ( create_client( mod_gm_opt->server_list, &client ) != GM_OK ) {
         gm_log( GM_LOG_ERROR, "cannot start client\n" );
         clean_worker_exit(0);
-        exit( EXIT_FAILURE );
+        _exit( EXIT_FAILURE );
     }
 
     /* create duplicate client */
     if ( create_client_dup( mod_gm_opt->dupserver_list, &client_dup ) != GM_OK ) {
         gm_log( GM_LOG_ERROR, "cannot start client for duplicate server\n" );
-        exit( EXIT_FAILURE );
+        _exit( EXIT_FAILURE );
     }
 
 #ifdef EMBEDDEDPERL
     if(init_embedded_perl(env) == GM_ERROR) {
-        exit( EXIT_FAILURE );
+        _exit( EXIT_FAILURE );
     }
 #endif
 
@@ -107,7 +107,7 @@ void worker_loop() {
         gearman_return_t ret;
 
         /* wait for a job, otherwise exit when hit the idle timeout */
-        if(mod_gm_opt->idle_timeout > 0 && worker_run_mode == GM_WORKER_MULTI) {
+        if(mod_gm_opt->idle_timeout > 0 && ( worker_run_mode == GM_WORKER_MULTI || worker_run_mode == GM_WORKER_STATUS )) {
             signal(SIGALRM, idle_sighandler);
             alarm(mod_gm_opt->idle_timeout);
         }
@@ -237,8 +237,16 @@ void *get_job( gearman_job_st *job, void *context, size_t *result_size, gearman_
         } else if ( !strcmp( key, "latency" ) ) {
             exec_job->latency = atof(value);
             valid_lines++;
+        } else if ( !strcmp( key, "next_check" ) ) {
+            string2timeval(value, &exec_job->next_check);
+            valid_lines++;
         } else if ( !strcmp( key, "start_time" ) ) {
-            string2timeval(value, &exec_job->core_start_time);
+            /* for compatibility reasons... (used by older mod-gearman neb modules) */
+            string2timeval(value, &exec_job->next_check);
+            string2timeval(value, &exec_job->core_time);
+            valid_lines++;
+        } else if ( !strcmp( key, "core_time" ) ) {
+            string2timeval(value, &exec_job->core_time);
             valid_lines++;
         } else if ( !strcmp( key, "timeout" ) ) {
             exec_job->timeout = atoi(value);
@@ -250,7 +258,7 @@ void *get_job( gearman_job_st *job, void *context, size_t *result_size, gearman_
     }
 
 #ifdef GM_DEBUG
-    if(exec_job->core_start_time.tv_sec < 10000)
+    if(exec_job->next_check.tv_sec < 10000)
         write_debug_file(&decrypted_orig);
 #endif
 
@@ -260,9 +268,6 @@ void *get_job( gearman_job_st *job, void *context, size_t *result_size, gearman_
         do_exec_job();
     }
 
-    /* send finish signal to parent */
-    set_state(GM_JOB_END);
-
     current_gearman_job = NULL;
 
     /* start listening to SIGTERMs */
@@ -272,10 +277,13 @@ void *get_job( gearman_job_st *job, void *context, size_t *result_size, gearman_
     free(decrypted_data_c);
     free_job(exec_job);
 
+    /* send finish signal to parent */
+    set_state(GM_JOB_END);
+
     if(mod_gm_opt->max_jobs > 0 && jobs_done >= mod_gm_opt->max_jobs) {
         gm_log( GM_LOG_TRACE, "jobs done: %i -> exiting...\n", jobs_done );
         clean_worker_exit(0);
-        exit( EXIT_SUCCESS );
+        _exit( EXIT_SUCCESS );
     }
 
     return NULL;
@@ -285,7 +293,7 @@ void *get_job( gearman_job_st *job, void *context, size_t *result_size, gearman_
 /* do some job */
 void do_exec_job( ) {
     struct timeval start_time, end_time;
-    int latency;
+    int latency, age;
 
     gm_log( GM_LOG_TRACE, "do_exec_job()\n" );
 
@@ -316,15 +324,22 @@ void do_exec_job( ) {
     /* get the check start time */
     gettimeofday(&start_time,NULL);
     exec_job->start_time = start_time;
-    latency = start_time.tv_sec - exec_job->core_start_time.tv_sec;
+    latency = start_time.tv_sec - exec_job->next_check.tv_sec;
+    age     = start_time.tv_sec - exec_job->core_time.tv_sec;
 
     gm_log( GM_LOG_TRACE, "timeout: %i, core latency: %i\n", exec_job->timeout, latency);
 
     /* job is too old */
-    if(latency > mod_gm_opt->max_age) {
+    if(mod_gm_opt->max_age > 0 && age > mod_gm_opt->max_age) {
         exec_job->return_code   = 3;
 
-        gm_log( GM_LOG_INFO, "discarded too old %s job: %i > %i\n", exec_job->type, (int)latency, mod_gm_opt->max_age);
+        if ( !strcmp( exec_job->type, "service" ) ) {
+            gm_log( GM_LOG_INFO, "discarded too old %s job: %i > %i (%s - %s)\n", exec_job->type, (int)age, mod_gm_opt->max_age, exec_job->host_name, exec_job->service_description);
+        } else if ( !strcmp( exec_job->type, "host" ) ) {
+            gm_log( GM_LOG_INFO, "discarded too old %s job: %i > %i (%s - %s)\n", exec_job->type, (int)age, mod_gm_opt->max_age, exec_job->host_name);
+        } else {
+            gm_log( GM_LOG_INFO, "discarded too old %s job: %i > %i\n", exec_job->type, (int)age, mod_gm_opt->max_age);
+        }
 
         gettimeofday(&end_time, NULL);
         exec_job->finish_time = end_time;
@@ -405,7 +420,7 @@ int set_worker( gearman_worker_st *w ) {
 void idle_sighandler(int sig) {
     gm_log( GM_LOG_TRACE, "idle_sighandler(%i)\n", sig );
     clean_worker_exit(0);
-    exit( EXIT_SUCCESS );
+    _exit( EXIT_SUCCESS );
 }
 
 
@@ -423,18 +438,26 @@ void set_state(int status) {
         perror("shmat");
         gm_log( GM_LOG_TRACE, "worker finished: %d\n", getpid() );
         clean_worker_exit(0);
-        exit( EXIT_FAILURE );
+        _exit( EXIT_FAILURE );
     }
 
     if(status == GM_JOB_START)
         shm[shm_index] = current_pid;
     if(status == GM_JOB_END) {
         shm[0]++; /* increase jobs done */
+
+        /* status slot changed to -1 -> exit */
+        if( shm[shm_index] == -1 ) {
+            gm_log( GM_LOG_TRACE, "worker finished: %d\n", getpid() );
+            clean_worker_exit(0);
+            _exit( EXIT_SUCCESS );
+        }
+
         /* pid in our status slot changed, this should not happen -> exit */
         if( shm[shm_index] != current_pid && shm[shm_index] != -current_pid ) {
             gm_log( GM_LOG_ERROR, "double used worker slot: %d != %d\n", current_pid, shm[shm_index] );
             clean_worker_exit(0);
-            exit( EXIT_FAILURE );
+            _exit( EXIT_FAILURE );
         }
         shm[shm_index] = -current_pid;
     }
@@ -477,15 +500,17 @@ void clean_worker_exit(int sig) {
     if((shm = shmat(shmid, NULL, 0)) == (int *) -1) {
         perror("shmat");
         gm_log( GM_LOG_TRACE, "worker finished: %d\n", getpid() );
-        exit( EXIT_FAILURE );
+        _exit( EXIT_FAILURE );
+    }
+    if( shm[shm_index] == current_pid || shm[shm_index] == -current_pid ) {
+        shm[shm_index] = -1;
     }
-    shm[shm_index] = -1;
 
     /* detach from shared memory */
     if(shmdt(shm) < 0)
         perror("shmdt");
 
-    exit( EXIT_SUCCESS );
+    _exit( EXIT_SUCCESS );
 }
 
 
